"""
App settings for fleetpings
"""

# Standard Library
from re import RegexFlag

# Django
from django.apps import apps
from django.conf import settings
from django.core.handlers.wsgi import WSGIRequest


def timezones_installed() -> bool:
    """
    Check if timezones is installed

    :return:
    :rtype:
    """

    return apps.is_installed(app_name="timezones")


def optimer_installed() -> bool:
    """
    Check if allianceauth.optimer is installed

    :return:
    :rtype:
    """

    return apps.is_installed(app_name="allianceauth.optimer")


def fittings_installed() -> bool:
    """
    Check if the Fittings module is installed

    :return:
    :rtype:
    """

    return apps.is_installed(app_name="fittings")


def discord_service_installed() -> bool:
    """
    Check if the Discord service module is installed

    :return:
    :rtype:
    """

    return apps.is_installed(app_name="allianceauth.services.modules.discord")


def srp_module_installed() -> bool:
    """
    Check if any SRP module is installed

    :return:
    :rtype:
    """

    return apps.is_installed(app_name="allianceauth.srp") or apps.is_installed(
        app_name="aasrp"
    )


def srp_module_is(module_name: str) -> bool:
    """
    Check if the given SRP module is installed

    :param module_name:
    :type module_name:
    :return:
    :rtype:
    """

    return apps.is_installed(app_name=module_name)


def can_add_srp_links(request: WSGIRequest, module_name: str) -> bool:
    """
    Check if the user can add SRP links

    :param request:
    :type request:
    :param module_name:
    :type module_name:
    :return:
    :rtype:
    """

    return_value = False

    if module_name == "aasrp" and (
        request.user.has_perm(perm="aasrp.manage_srp")
        or request.user.has_perm(perm="aasrp.create_srp")
    ):
        return_value = True

    if module_name == "allianceauth.srp" and (
        request.user.has_perm(perm="auth.srp_management")
        or request.user.has_perm(perm="srp.add_srpfleetmain")
    ):
        return_value = True

    return return_value


def use_fittings_module_for_doctrines() -> bool:
    """
    Check if the Fittings module is used for doctrines

    :return:
    :rtype:
    """

    # AA Fleet Pings
    from fleetpings.models import (  # pylint: disable=import-outside-toplevel, cyclic-import
        Setting,
    )

    return (
        fittings_installed() is True
        and Setting.objects.get_setting(
            setting_key=Setting.Field.USE_DOCTRINES_FROM_FITTINGS_MODULE
        )
        is True
    )


def debug_enabled() -> RegexFlag:
    """
    Check if DEBUG is enabled

    :return:
    :rtype:
    """

    return settings.DEBUG



"""
AA Fleet Pings :: Admin
"""

# Third Party
from solo.admin import SingletonModelAdmin

# Django
from django.contrib import admin
from django.contrib.auth.models import Group
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

# AA Fleet Pings
from fleetpings.form import FleetTypeAdminForm, SettingAdminForm
from fleetpings.models import (
    DiscordPingTarget,
    FleetComm,
    FleetDoctrine,
    FleetType,
    FormupLocation,
    Setting,
    Webhook,
)


def custom_filter(title) -> admin.FieldListFilter:
    """
    Custom filter for model properties

    :param title:
    :type title:
    :return:
    :rtype:
    """

    class Wrapper(admin.FieldListFilter):
        """
        Wrapper
        """

        def __new__(cls, *args, **kwargs):
            instance = admin.FieldListFilter.create(*args, **kwargs)
            instance.title = title

            return instance

    return Wrapper


@admin.register(FleetComm)
class FleetCommAdmin(admin.ModelAdmin):
    """
    FleetCommAdmin
    """

    list_display = ("_name", "channel", "notes", "is_enabled")
    ordering = ("name", "channel")
    list_filter = ("is_enabled",)

    @classmethod
    @admin.display(description=_("Fleet comm"), ordering="name")
    def _name(cls, obj: FleetComm) -> str:
        """
        _name

        :param obj:
        :type obj:
        :return:
        :rtype:
        """

        return obj.name


@admin.register(FleetDoctrine)
class FleetDoctrineAdmin(admin.ModelAdmin):
    """
    FleetDoctrineAdmin
    """

    list_display = ("_name", "_link", "_restricted_to_group", "notes", "is_enabled")
    filter_horizontal = ("restricted_to_group",)
    ordering = ("name",)

    list_filter = (
        ("is_enabled", custom_filter(title="active")),
        ("restricted_to_group", custom_filter(title="restriction")),
    )

    @classmethod
    @admin.display(description=_("Doctrine"), ordering="name")
    def _name(cls, obj: FleetDoctrine) -> str:
        """
        _name

        :param obj:
        :type obj:
        :return:
        :rtype:
        """

        return obj.name

    @classmethod
    @admin.display(description=_("Doctrine link"), ordering="link")
    def _link(cls, obj: FleetDoctrine) -> str:
        """
        _link

        :param obj:
        :type obj:
        :return:
        :rtype:
        """

        return obj.name

    @classmethod
    @admin.display(
        description=_("Group restrictions"), ordering="restricted_to_group__name"
    )
    def _restricted_to_group(cls, obj: FleetDoctrine) -> str | None:
        """
        _restricted_to_group

        :param obj:
        :type obj:
        :return:
        :rtype:
        """

        names = [x.name for x in obj.restricted_to_group.all().order_by("name")]

        if names:
            return ", ".join(names)

        return None


@admin.register(FormupLocation)
class FormupLocationAdmin(admin.ModelAdmin):
    """
    FormupLocationAdmin
    """

    list_display = ("name", "notes", "is_enabled")
    ordering = ("name",)
    list_filter = ("is_enabled",)


@admin.register(DiscordPingTarget)
class DiscordPingTargetsAdmin(admin.ModelAdmin):
    """
    DiscordPingTargetsAdmin
    """

    list_display = (
        "_name",
        "discord_id",
        "_restricted_to_group",
        "notes",
        "is_enabled",
    )

    filter_horizontal = ("restricted_to_group",)
    readonly_fields = ("discord_id",)
    ordering = ("name",)

    list_filter = (
        ("is_enabled", custom_filter(title="active")),
        ("name", custom_filter(title="target")),
        ("restricted_to_group", custom_filter(title="restriction")),
    )

    @classmethod
    @admin.display(description=_("Ping target"), ordering="name")
    def _name(cls, obj: DiscordPingTarget) -> Group:
        """
        _name

        :param obj:
        :type obj:
        :return:
        :rtype:
        """

        return obj.name

    @classmethod
    @admin.display(
        description=_("Group restrictions"), ordering="restricted_to_group__name"
    )
    def _restricted_to_group(cls, obj: DiscordPingTarget) -> str | None:
        """
        _restricted_to_group

        :param obj:
        :type obj:
        :return:
        :rtype:
        """

        names = [x.name for x in obj.restricted_to_group.all().order_by("name")]

        if names:
            return ", ".join(names)

        return None


@admin.register(FleetType)
class FleetTypeAdmin(admin.ModelAdmin):
    """
    FleetTypeAdmin
    """

    form = FleetTypeAdminForm

    list_display = (
        "_name",
        "_embed_color",
        "_restricted_to_group",
        "notes",
        "is_enabled",
    )

    filter_horizontal = ("restricted_to_group",)
    ordering = ("name",)

    list_filter = (
        ("is_enabled", custom_filter(title="active")),
        ("restricted_to_group", custom_filter(title="restriction")),
    )

    @classmethod
    @admin.display(description=_("Fleet type"), ordering="name")
    def _name(cls, obj: FleetType) -> str:
        """
        _name

        :param obj:
        :type obj:
        :return:
        :rtype:
        """

        return obj.name

    @classmethod
    @admin.display(description=_("Embed color"), ordering="embed_color")
    def _embed_color(cls, obj: FleetType) -> str:
        """
        _embed_color

        :param obj:
        :type obj:
        :return:
        :rtype:
        """

        return_value = (
            "<span "
            'style="display: inline-block; width: 16px; background-color: {bg_color};">'
            "&nbsp;&nbsp;"
            "</span> {bg_color}".format(bg_color=obj.embed_color)
        )

        return mark_safe(return_value)

    @classmethod
    @admin.display(
        description=_("Group restrictions"), ordering="restricted_to_group__name"
    )
    def _restricted_to_group(cls, obj: FleetType) -> str | None:
        names = [x.name for x in obj.restricted_to_group.all().order_by("name")]

        if names:
            return ", ".join(names)

        return None


@admin.register(Webhook)
class WebhookAdmin(admin.ModelAdmin):
    """
    WebhookAdmin
    """

    list_display = (
        "_name",
        "_url",
        "_restricted_to_group",
        "notes",
        "is_enabled",
    )

    filter_horizontal = ("restricted_to_group",)
    ordering = ("name",)

    list_filter = (
        ("is_enabled", custom_filter(title="active")),
        ("restricted_to_group", custom_filter(title="restriction")),
    )

    @classmethod
    @admin.display(description=_("Discord channel"), ordering="name")
    def _name(cls, obj: Webhook) -> str:
        return obj.name

    @classmethod
    @admin.display(description=_("Webhook URL"), ordering="url")
    def _url(cls, obj: Webhook) -> str:
        return obj.url

    @classmethod
    @admin.display(
        description=_("Group restrictions"), ordering="restricted_to_group__name"
    )
    def _restricted_to_group(cls, obj: Webhook) -> str | None:
        names = [x.name for x in obj.restricted_to_group.all().order_by("name")]

        if names:
            return ", ".join(names)

        return None


@admin.register(Setting)
class SettingAdmin(SingletonModelAdmin):
    """
    Setting Admin
    """

    form = SettingAdminForm



# Generated by Django 4.0.10 on 2023-06-23 10:49

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("fleetpings", "0013_fleetcomm_channel"),
    ]

    operations = [
        migrations.AlterField(
            model_name="discordpingtargets",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict ping rights to the following groups …",
                related_name="discord_role_require_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.AlterField(
            model_name="fleetdoctrine",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict this doctrine to the following groups …",
                related_name="fleetdoctrine_require_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.AlterField(
            model_name="fleettype",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict this fleet type to the following groups …",
                related_name="fleettype_require_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict ping rights to the following groups …",
                related_name="webhook_require_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.RenameModel(
            old_name="DiscordPingTargets",
            new_name="DiscordPingTarget",
        ),
    ]



"""
Testing the template tags
"""

# Django
from django.template import Context, Template
from django.test import TestCase

# Alliance Auth (External Libs)
from app_utils.urls import site_absolute_url


class TestReverseUrl(TestCase):
    """
    Test fleetpings_reverse_url template tag
    """

    def test_reverse_url(self):
        """
        Test should return a reversed URL

        :return:
        :rtype:
        """

        context = Context(dict_={"doctrine_pk": "1"})
        template_to_render = Template(
            template_string=(
                "{% load fleetpings %}"
                "{% fleetpings_reverse_url 'fittings:view_doctrine' doctrine_pk %}"
            )
        )

        rendered_template = template_to_render.render(context=context)
        site_url = site_absolute_url()

        self.assertInHTML(
            needle=f'{site_url}/fittings/doctrine/{context["doctrine_pk"]}/',
            haystack=rendered_template,
        )



"""
Models for the Fleet Pings app
"""

# Standard Library
import re
from typing import ClassVar

# Third Party
from requests.exceptions import HTTPError
from solo.models import SingletonModel

# Django
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.core.validators import URLValidator
from django.db import models
from django.utils.translation import gettext_lazy as _

# AA Fleet Pings
from fleetpings.app_settings import discord_service_installed
from fleetpings.constants import DISCORD_WEBHOOK_REGEX
from fleetpings.managers import SettingManager

# Check if the Discord service is active
if discord_service_installed():
    # Alliance Auth
    from allianceauth.services.modules.discord.models import DiscordUser


def _get_discord_group_info(ping_target: Group) -> dict:
    """
    Get the Discord group info for the given group

    :param ping_target:
    :type ping_target:
    :return:
    :rtype:
    """

    if not discord_service_installed():
        raise ValidationError(
            message=_("You might want to install the Discord service first …")
        )

    try:
        discord_group_info = DiscordUser.objects.group_to_role(  # pylint: disable=possibly-used-before-assignment
            group=ping_target
        )
    except HTTPError as http_error:
        raise ValidationError(
            message=_(
                "Are you sure you have your Discord linked to your Alliance Auth?"
            )
        ) from http_error

    if not discord_group_info:
        raise ValidationError(
            message=_("This group has not been synced to Discord yet.")
        )

    return discord_group_info


class AaFleetpings(models.Model):
    """
    Alliance Auth Fleet Pings
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        AaFleetpings :: Meta
        """

        managed = False
        default_permissions = ()
        permissions = (("basic_access", _("Can access this app")),)


class FleetComm(models.Model):
    """
    Fleet Comms
    """

    name = models.CharField(
        max_length=255,
        help_text=_("Short name to identify this comms"),
        verbose_name=_("Name"),
    )

    channel = models.CharField(
        blank=True,
        max_length=255,
        help_text=_("In which channel is the fleet?"),
        verbose_name=_("Channel"),
    )

    notes = models.TextField(
        null=True,
        blank=True,
        help_text=_("You can add notes about this configuration here if you want"),
        verbose_name=_("Notes"),
    )

    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_("Whether this comms is enabled or not"),
        verbose_name=_("Is enabled"),
    )

    def __str__(self) -> str:
        """
        String representation of the object

        :return:
        :rtype:
        """

        return f"{self.name} » {self.channel}" if self.channel else f"{self.name}"

    class Meta:  # pylint: disable=too-few-public-methods
        """
        FleetComm :: Meta
        """

        verbose_name = _("Fleet comm")
        verbose_name_plural = _("Fleet comms")
        default_permissions = ()

        unique_together = ("name", "channel")


class FleetDoctrine(models.Model):
    """
    Fleet Doctrines
    """

    # Doctrine name
    name = models.CharField(
        max_length=255,
        unique=True,
        help_text=_("Short name to identify this doctrine"),
        verbose_name=_("Name"),
    )

    # Link to your doctrine
    link = models.CharField(
        max_length=255,
        blank=True,
        help_text=_("A link to a doctrine page for this doctrine if you have."),
        verbose_name=_("Doctrine link"),
    )

    # Restrictions
    restricted_to_group = models.ManyToManyField(
        to=Group,
        blank=True,
        related_name="fleetdoctrine_require_groups",
        help_text=_("Restrict this doctrine to the following groups …"),
        verbose_name=_("Group restrictions"),
    )

    # Doctrine notes
    notes = models.TextField(
        null=True,
        blank=True,
        help_text=_("You can add notes about this configuration here if you want"),
        verbose_name=_("Notes"),
    )

    # Is doctrine active
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_("Whether this doctrine is enabled or not"),
        verbose_name=_("Is enabled"),
    )

    def clean(self):
        """
        Check if the doctrine link is valid

        :return:
        :rtype:
        """

        doctrine_link = self.link

        if doctrine_link != "":
            validate = URLValidator()

            try:
                validate(doctrine_link)
            except ValidationError as exception:
                raise ValidationError(
                    message=_("Your doctrine URL is not valid.")
                ) from exception

        super().clean()

    def __str__(self) -> str:
        """
        String representation of the object

        :return:
        :rtype:
        """

        return str(self.name)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        FleetDoctrine :: Meta
        """

        verbose_name = _("Fleet doctrine")
        verbose_name_plural = _("Fleet doctrines")
        default_permissions = ()


class FormupLocation(models.Model):
    """
    Formup Location
    """

    # formup location name
    name = models.CharField(
        max_length=255,
        unique=True,
        help_text=_("Short name to identify this formup location"),
        verbose_name=_("Name"),
    )

    # Formup location notes
    notes = models.TextField(
        null=True,
        blank=True,
        help_text=_("You can add notes about this configuration here if you want"),
        verbose_name=_("Notes"),
    )

    # Is formup location active?
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_("Whether this formup location is enabled or not"),
        verbose_name=_("Is enabled"),
    )

    def __str__(self) -> str:
        """
        String representation of the object

        :return:
        :rtype:
        """
        return str(self.name)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        FormupLocation :: Meta
        """

        verbose_name = _("Formup location")
        verbose_name_plural = _("Formup locations")
        default_permissions = ()


class DiscordPingTarget(models.Model):
    """
    Discord Ping Targets
    """

    # Discord group to ping
    name = models.OneToOneField(
        to=Group,
        on_delete=models.CASCADE,
        unique=True,
        help_text=(
            _(
                "Name of the Discord role to ping. "
                "(Note: This must be an Auth group that is synced to Discord.)"
            )
        ),
        verbose_name=_("Group name"),
    )

    # Discord group id
    discord_id = models.CharField(
        max_length=255,
        unique=True,
        blank=True,
        help_text=_("ID of the Discord role to ping"),
        verbose_name=_("Discord ID"),
    )

    # Restrictions
    restricted_to_group = models.ManyToManyField(
        to=Group,
        blank=True,
        related_name="discord_role_require_groups",
        help_text=_("Restrict ping rights to the following groups …"),
        verbose_name=_("Group restrictions"),
    )

    # Notes
    notes = models.TextField(
        null=True,
        blank=True,
        help_text=_("You can add notes about this configuration here if you want"),
        verbose_name=_("Notes"),
    )

    # Is this group active?
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_("Whether this formup location is enabled or not"),
        verbose_name=_("Is enabled"),
    )

    def clean(self):
        """
        Check if the Discord group exists and get the Discord group name

        :return:
        :rtype:
        """

        _get_discord_group_info(self.name)

        super().clean()

    def save(
        self,
        force_insert=False,  # pylint: disable=unused-argument
        force_update=False,  # pylint: disable=unused-argument
        using=None,  # pylint: disable=unused-argument
        update_fields=None,  # pylint: disable=unused-argument
    ):
        """
        Save action

        :param force_insert:
        :type force_insert:
        :param force_update:
        :type force_update:
        :param using:
        :type using:
        :param update_fields:
        :type update_fields:
        :return:
        :rtype:
        """

        # Check if the Discord service is active
        if discord_service_installed():
            discord_group_info = _get_discord_group_info(self.name)
            self.discord_id = discord_group_info["id"]

        super().save()  # Call the "real" save() method.

    def __str__(self) -> str:
        """
        String representation of the object

        :return:
        :rtype:
        """

        return str(self.name)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        DiscordPingTargets :: Meta
        """

        verbose_name = _("Discord ping target")
        verbose_name_plural = _("Discord ping targets")
        default_permissions = ()


class FleetType(models.Model):
    """
    Fleet Types
    """

    # Name of the fleet type
    name = models.CharField(
        max_length=255,
        unique=True,
        help_text=_("Short name to identify this fleet type"),
        verbose_name=_("Name"),
    )

    # Embed color
    embed_color = models.CharField(
        max_length=7,
        blank=True,
        help_text=_("Highlight color for the embed"),
        verbose_name=_("Embed color"),
    )

    # Restrictions
    restricted_to_group = models.ManyToManyField(
        to=Group,
        blank=True,
        related_name="+",
        help_text=_("Restrict this fleet type to the following groups …"),
        verbose_name=_("Group restrictions"),
    )

    # Fleet type notes
    notes = models.TextField(
        null=True,
        blank=True,
        help_text=_("You can add notes about this configuration here if you want"),
        verbose_name=_("Notes"),
    )

    # Is this fleet type enabled
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_("Whether this fleet type is enabled or not"),
        verbose_name=_("Is enabled"),
    )

    def __str__(self) -> str:
        """
        String representation of the object

        :return:
        :rtype:
        """

        return str(self.name)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        FleetType :: Meta
        """

        verbose_name = _("Fleet type")
        verbose_name_plural = _("Fleet types")
        default_permissions = ()


class Webhook(models.Model):
    """
    A Discord webhook
    """

    # Channel name
    name = models.CharField(
        max_length=255,
        unique=True,
        help_text=_("Name of the channel this webhook posts to"),
        verbose_name=_("Discord channel"),
    )

    # Wehbook url
    url = models.CharField(
        max_length=255,
        unique=True,
        help_text=(
            _(
                "URL of this webhook, e.g. "
                "https://discord.com/api/webhooks/123456/abcdef"
            )
        ),
        verbose_name=_("Webhook URL"),
    )

    # Restrictions
    restricted_to_group = models.ManyToManyField(
        to=Group,
        blank=True,
        related_name="webhook_require_groups",
        help_text=_("Restrict ping rights to the following groups …"),
        verbose_name=_("Group restrictions"),
    )

    # Webhook notes
    notes = models.TextField(
        null=True,
        blank=True,
        help_text=_("You can add notes about this webhook here if you want"),
        verbose_name=_("Notes"),
    )

    # Is it enabled?
    is_enabled = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_("Whether this webhook is active or not"),
        verbose_name=_("Is enabled"),
    )

    def __str__(self) -> str:
        """
        String representation of the object

        :return:
        :rtype:
        """

        return str(self.name)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Webhook :: Meta
        """

        verbose_name = _("Webhook")
        verbose_name_plural = _("Webhooks")
        default_permissions = ()

    def clean(self):
        """
        Check if the webhook URL is valid

        :return:
        :rtype:
        """

        # Check if it's an actual Discord Webhook URL if the verification setting is set.
        if not re.match(
            pattern=DISCORD_WEBHOOK_REGEX, string=self.url
        ) and Setting.objects.get_setting(
            setting_key=Setting.Field.WEBHOOK_VERIFICATION
        ):
            raise ValidationError(
                message=_(
                    "Invalid webhook URL. The webhook URL you entered does not match "
                    "any known format for a Discord webhook. Please check the "
                    "webhook URL."
                )
            )

        super().clean()


class Setting(SingletonModel):
    """
    Default forum settings
    """

    class Field(models.TextChoices):
        """
        Choices for Setting.Field
        """

        USE_DEFAULT_FLEET_TYPES = "use_default_fleet_types", _(
            "Use default fleet types"
        )
        USE_DEFAULT_PING_TARGETS = "use_default_ping_targets", _(
            "Use default ping targets"
        )
        USE_DOCTRINES_FROM_FITTINGS_MODULE = "use_doctrines_from_fittings_module", _(
            "Use Doctrines from Fittings module"
        )
        WEBHOOK_VERIFICATION = "webhook_verification", _("Verify webhooks")
        DEFAULT_EMBED_COLOR = "default_embed_color", _("Default embed color")

    use_default_fleet_types = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_(
            "Whether to use default fleet types. If checked, the default fleet types "
            "(Roaming, Home Defense, StratOP, and CTA) will be added to the Fleet Type "
            "dropdown."
        ),
        verbose_name=Field.USE_DEFAULT_FLEET_TYPES.label,  # pylint: disable=no-member
    )

    use_default_ping_targets = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_(
            "Whether to use default ping targets. If checked, the default ping targets "
            "(@everyone and @here) will be added to the Ping Target dropdown."
        ),
        verbose_name=Field.USE_DEFAULT_PING_TARGETS.label,  # pylint: disable=no-member
    )

    use_doctrines_from_fittings_module = models.BooleanField(
        default=False,
        db_index=True,
        help_text=_(
            "Whether to use the doctrines from the Fittings modules in the doctrine "
            "dropdown. Note: The fittings module needs to be installed for this."
        ),
        verbose_name=Field.USE_DOCTRINES_FROM_FITTINGS_MODULE.label,  # pylint: disable=no-member
    )

    webhook_verification = models.BooleanField(
        default=True,
        db_index=True,
        help_text=_(
            "Whether to verify webhooks URLs or not. Note: When unchecked, webhook URLs "
            "will not be verified, so the app can be used with non-Discord webhooks "
            "as well. When disabling webhook verification and using non-Discord "
            "webhooks, it is up to you to make sure your webhook understands a payload "
            "that is formatted for Discord webhooks."
        ),
        verbose_name=Field.WEBHOOK_VERIFICATION.label,  # pylint: disable=no-member
    )

    default_embed_color = models.CharField(
        default="#FAA61A",
        max_length=7,
        blank=True,
        help_text=_("Default highlight color for the webhook embed."),
        verbose_name=Field.DEFAULT_EMBED_COLOR.label,  # pylint: disable=no-member
    )

    objects: ClassVar[SettingManager] = SettingManager()

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Setting :: Meta
        """

        default_permissions = ()
        verbose_name = _("Setting")
        verbose_name_plural = _("Settings")

    def __str__(self) -> str:
        """
        String representation of the object

        :return:
        :rtype:
        """

        return str(_("Fleet Pings Settings"))



# Generated by Django 3.1.5 on 2021-01-14 17:33

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("fleetpings", "0007_mysql8_default_value_fixes_20201003_1232"),
    ]

    operations = [
        migrations.AlterField(
            model_name="webhook",
            name="url",
            field=models.CharField(
                help_text=(
                    "URL of this webhook, e.g. "
                    "https://discord.com/api/webhooks/123456/abcdef or "
                    "https://hooks.slack.com/services/xxxx/xxxx"
                ),
                max_length=255,
                unique=True,
            ),
        ),
    ]



"""
Tests for model FleetDoctrine
"""

# Django
from django.core.exceptions import ValidationError
from django.test import TestCase

# AA Fleet Pings
from fleetpings.models import FleetDoctrine


class TestModelFleetDoctrine(TestCase):
    """
    Testing the FleetDoctrine model
    """

    def test_should_return_fleetdoctrine_model_string_name(self):
        """
        Test should return FleetDoctrine model string name

        :return:
        :rtype:
        """

        doctrine = FleetDoctrine(name="Awesome Ships")
        doctrine.save()

        self.assertEqual(first=str(doctrine), second="Awesome Ships")

    def test_doctrine_link_should_throw_exception(self):
        """
        Test if we get a ValidationError for a doctrine link that is not valid

        :return:
        :rtype:
        """

        # given
        doctrine = FleetDoctrine(
            name="Awesome Doctrine", link="htp://invalid-doctrine.url"
        )

        # when
        with self.assertRaises(expected_exception=ValidationError):
            doctrine.clean()

        with self.assertRaisesMessage(
            expected_exception=ValidationError,
            expected_message="Your doctrine URL is not valid.",
        ):
            doctrine.clean()



"""
Hook into AA
"""

# Alliance Auth
from allianceauth import hooks
from allianceauth.services.hooks import MenuItemHook, UrlHook

# AA Fleet Pings
from fleetpings import __title__, urls


class AaFleetpingsMenuItem(MenuItemHook):  # pylint: disable=too-few-public-methods
    """
    This class ensures only authorized users will see the menu entry
    """

    def __init__(self):
        """
        Constructor
        """

        # Setup menu entry for sidebar
        MenuItemHook.__init__(
            self,
            text=__title__,
            classes="fa-regular fa-bell",
            url_name="fleetpings:index",
            navactive=["fleetpings:index"],
        )

    def render(self, request):
        """
        Render the menu item

        :param request:
        :type request:
        :return:
        :rtype:
        """

        if request.user.has_perm(perm="fleetpings.basic_access"):
            return MenuItemHook.render(self, request=request)

        return ""


@hooks.register("menu_item_hook")
def register_menu():
    """
    Register our menu entry

    :return:
    :rtype:
    """

    return AaFleetpingsMenuItem()


@hooks.register("url_hook")
def register_urls():
    """
    Register our URLs

    :return:
    :rtype:
    """

    return UrlHook(urls=urls, namespace="fleetpings", base_url=r"^fleetpings/")



"""
Test checks for access to fleetpings
"""

# Standard Library
from http import HTTPStatus

# Django
from django.contrib.auth.models import Group
from django.test import TestCase, modify_settings
from django.urls import reverse

# AA Fleet Pings
from fleetpings.tests.utils import create_fake_user


class TestAccess(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        """
        Setup

        :return:
        :rtype:
        """

        super().setUpClass()

        cls.group = Group.objects.create(name="Superhero")

        # User cannot access fleetpings
        cls.user_1001 = create_fake_user(
            character_id=1001, character_name="Peter Parker"
        )

        # User can access fleetpings
        cls.user_1002 = create_fake_user(
            character_id=1002,
            character_name="Bruce Wayne",
            permissions=["fleetpings.basic_access"],
        )

        # User can add srp (aasrp)
        cls.user_1003 = create_fake_user(
            character_id=1003,
            character_name="Clark Kent",
            permissions=["fleetpings.basic_access", "aasrp.create_srp"],
        )

    def test_has_no_access(self):
        """
        Test that a user without access gets redirected

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1001)

        # when
        res = self.client.get(path=reverse(viewname="fleetpings:index"))

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)

    def test_has_access(self):
        """
        Test that a user with access gets to see it

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1002)

        # when
        res = self.client.get(path=reverse(viewname="fleetpings:index"))

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.OK)

    @modify_settings(
        INSTALLED_APPS={
            "remove": [
                "allianceauth.srp",
                "aasrp",
                "timezones",
                "allianceauth.optimer",
                "fittings",
            ]
        }
    )
    def test_has_access_without_additional_modules(self):
        """
        Test that a user with access gets to see it (without additional modules)

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1002)

        # when
        response = self.client.get(path=reverse(viewname="fleetpings:index"))

        # then
        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)

        # context data
        self.assertEqual(first=response.context["optimer_installed"], second=False)
        self.assertEqual(first=response.context["fittings_installed"], second=False)
        self.assertEqual(
            first=response.context["srp_module_available_to_user"], second=False
        )

    @modify_settings(
        INSTALLED_APPS={
            "remove": [
                "allianceauth.srp",
                "aasrp",
                "timezones",
                "allianceauth.optimer",
                "fittings",
            ],
            "append": "aasrp",
        }
    )
    def test_has_access_with_aasrp(self):
        """
        Test that a user with access gets to see it (aasrp access)

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1003)

        # when
        response = self.client.get(path=reverse(viewname="fleetpings:index"))

        # then
        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)

        # context data
        self.assertEqual(first=response.context["optimer_installed"], second=False)
        self.assertEqual(first=response.context["fittings_installed"], second=False)
        self.assertEqual(
            first=response.context["srp_module_available_to_user"], second=True
        )

    @modify_settings(
        INSTALLED_APPS={
            "remove": [
                "allianceauth.srp",
                "aasrp",
                "timezones",
                "allianceauth.optimer",
                "fittings",
            ],
            "append": "allianceauth.srp",
        }
    )
    def test_has_access_with_allianceauth_srp(self):
        """
        Test that a user with access gets to see it (allianceauth.srp access)

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1003)

        # when
        response = self.client.get(path=reverse(viewname="fleetpings:index"))

        # then
        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)

        # context data
        self.assertEqual(first=response.context["optimer_installed"], second=False)
        self.assertEqual(first=response.context["fittings_installed"], second=False)
        self.assertEqual(
            first=response.context["srp_module_available_to_user"], second=True
        )

    @modify_settings(
        INSTALLED_APPS={
            "remove": [
                "allianceauth.srp",
                "aasrp",
                "timezones",
                "allianceauth.optimer",
                "fittings",
            ],
            "append": "timezones",
        }
    )
    def test_has_access_with_timezones(self):
        """
        Test that a user with access gets to see it (timezones access)

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1003)

        # when
        response = self.client.get(path=reverse(viewname="fleetpings:index"))

        # then
        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)

        # context data
        self.assertEqual(first=response.context["optimer_installed"], second=False)
        self.assertEqual(first=response.context["fittings_installed"], second=False)
        self.assertEqual(
            first=response.context["srp_module_available_to_user"], second=False
        )

    @modify_settings(
        INSTALLED_APPS={
            "remove": [
                "allianceauth.srp",
                "aasrp",
                "timezones",
                "allianceauth.optimer",
                "fittings",
            ],
            "append": "allianceauth.optimer",
        }
    )
    def test_has_access_with_allianceauth_optimer(self):
        """
        Test that a user with access gets to see it (allianceauth.optimer access)

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1003)

        # when
        response = self.client.get(path=reverse(viewname="fleetpings:index"))

        # then
        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)

        # context data
        self.assertEqual(first=response.context["optimer_installed"], second=True)
        self.assertEqual(first=response.context["fittings_installed"], second=False)
        self.assertEqual(
            first=response.context["srp_module_available_to_user"], second=False
        )



"""
Tests for model FleetType
"""

# Django
from django.test import TestCase

# AA Fleet Pings
from fleetpings.models import FleetType


class TestModelFleetType(TestCase):
    """
    Testing the FleetType model
    """

    def test_should_return_fleettype_model_string_name(self):
        """
        Test should return FleetType model string name

        :return:
        :rtype:
        """

        fleet_type = FleetType(name="Fun Fleet")
        fleet_type.save()

        self.assertEqual(first=str(fleet_type), second="Fun Fleet")



"""
Tests for the eve_images helper
"""

# Django
from django.test import TestCase

# Alliance Auth (External Libs)
from app_utils.testing import create_fake_user

# AA Fleet Pings
from fleetpings.helper.eve_images import get_character_portrait_from_evecharacter


class TestHelperEveImages(TestCase):
    """
    Test eve_images helper
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Setup

        :return:
        :rtype:
        """

        super().setUpClass()

        cls.user_1001 = create_fake_user(
            character_id=1001, character_name="Peter Parker"
        )

    def test_should_return_character_portrait_url(self):
        """
        Test should return character portrait URL

        :return:
        :rtype:
        """

        character = self.user_1001.profile.main_character

        portrait_url = get_character_portrait_from_evecharacter(character=character)
        expected_url = f"https://images.evetech.net/characters/{character.character_id}/portrait?size=32"  # pylint: disable=line-too-long

        self.assertEqual(first=portrait_url, second=expected_url)

    def test_should_return_character_portrait_html(self):
        """
        Test should return character portrait HTML

        :return:
        :rtype:
        """

        character = self.user_1001.profile.main_character

        portrait_html = get_character_portrait_from_evecharacter(
            character=character, as_html=True
        )
        expected_url = f"https://images.evetech.net/characters/{character.character_id}/portrait?size=32"  # pylint: disable=line-too-long
        expected_html = (
            '<img class="aa-fleetpings-character-portrait img-rounded" '
            f'src="{expected_url}" alt="{character.character_name}" '
            'width="32" height="32">'
        )

        self.assertEqual(first=portrait_html, second=expected_html)



# Django
from django.db import migrations

# Alliance Auth (External Libs)
from app_utils.app_settings import clean_setting

use_doctrines_from_fittings_module = clean_setting(
    "AA_FLEETPINGS_USE_DOCTRINES_FROM_FITTINGS_MODULE", False
)

webhook_verification = clean_setting("AA_FLEETPINGS_WEBHOOK_VERIFICATION", True)


def on_migrate(apps, schema_editor):
    """
    Create default settings on migration
    :param apps:
    :param schema_editor:
    :return:
    """

    Setting = apps.get_model("fleetpings", "Setting")
    db_alias = schema_editor.connection.alias

    Setting.objects.using(db_alias).create(
        pk=1,
        use_doctrines_from_fittings_module=use_doctrines_from_fittings_module,
        webhook_verification=webhook_verification,
    )


def on_migrate_zero(apps, schema_editor):
    """
    Remove default settings on migration to zero
    :param apps:
    :param schema_editor:
    :return:
    """

    Setting = apps.get_model("fleetpings", "Setting")
    db_alias = schema_editor.connection.alias
    Setting.objects.using(db_alias).delete()


class Migration(migrations.Migration):
    """
    Run migrations
    """

    dependencies = [
        ("fleetpings", "0011_settings_and_verbose_names"),
    ]

    operations = [migrations.RunPython(on_migrate, on_migrate_zero)]



"""
Managers for the fleetpings app
"""

# Django
from django.db import models


class SettingQuerySet(models.QuerySet):
    """
    SettingQuerySet
    """

    def delete(self):
        """
        Delete action

        Override:   We don't allow deletion here, so we make sure the object
                    is saved again and not deleted
        :return:
        """

        return super().update()


class SettingManager(models.Manager):
    """
    SettingManager
    """

    def get_setting(self, setting_key: str) -> str:
        """
        Get a setting

        :param setting_key:
        :type setting_key:
        :return:
        :rtype:
        """

        return getattr(self.first(), setting_key)

    def get_queryset(self):
        """
        Get the queryset

        :return:
        :rtype:
        """

        return SettingQuerySet(self.model)



# Generated by Django 4.0.10 on 2023-05-30 16:51

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("fleetpings", "0012_migrate_app_settings_from_local_py_to_database"),
    ]

    operations = [
        migrations.AddField(
            model_name="fleetcomm",
            name="channel",
            field=models.CharField(
                blank=True,
                help_text="In which channel is the fleet?",
                max_length=255,
                verbose_name="Channel",
            ),
        ),
        migrations.AlterField(
            model_name="fleetcomm",
            name="name",
            field=models.CharField(
                help_text="Short name to identify this comms",
                max_length=255,
                verbose_name="Name",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="fleetcomm",
            unique_together={("name", "channel")},
        ),
    ]



"""
Tests for model FleetComm
"""

# Django
from django.db.utils import IntegrityError
from django.test import TestCase

# AA Fleet Pings
from fleetpings.models import FleetComm


class TestModelFleetComm(TestCase):
    """
    Testing FleetComm model
    """

    def test_should_return_fleetcomm_model_string_name(self):
        """
        Test should return FleetComm model string name

        :return:
        :rtype:
        """

        test_object = FleetComm(name="Alliance Mumble")
        test_object_with_channel = FleetComm(name="Alliance Mumble", channel="Fleet 1")

        test_object.save()
        test_object_with_channel.save()

        self.assertEqual(first=str(test_object), second="Alliance Mumble")
        self.assertEqual(
            first=str(test_object_with_channel), second="Alliance Mumble » Fleet 1"
        )

    def test_should_throw_integrity_error_for_duplicate_comms_without_channel(self):
        """
        Test should throw an IntegrityError for duplicate comms without a channel

        :return:
        :rtype:
        """

        FleetComm(name="Alliance Mumble").save()

        new_fleetcomm = FleetComm(name="Alliance Mumble")

        with self.assertRaises(expected_exception=IntegrityError):
            new_fleetcomm.save()

    def test_should_throw_integrity_error_for_duplicate_comms_with_channel(self):
        """
        Test should throw an IntegrityError for duplicate comms with a channel

        :return:
        :rtype:
        """

        FleetComm(name="Alliance Mumble", channel="Fleet 1").save()

        new_fleetcomm = FleetComm(name="Alliance Mumble", channel="Fleet 1")

        with self.assertRaises(expected_exception=IntegrityError):
            new_fleetcomm.save()



"""
App config
"""

# Django
from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _

# AA Fleet Pings
from fleetpings import __version__


class AaFleetpingsConfig(AppConfig):
    """
    Application config
    """

    name = "fleetpings"
    label = "fleetpings"
    # Translators: This is the app name and version, which will appear in the Django Backend
    verbose_name = _(f"Fleet Pings v{__version__}")



"""
Test cases for the `fleetpings.helper.discord_webhook` module.
"""

# Standard Library
from unittest import TestCase

# AA Fleet Pings
from fleetpings import __version__
from fleetpings.constants import APP_NAME, GITHUB_URL
from fleetpings.helper.discord_webhook import get_user_agent


class TestUserAgent(TestCase):
    """
    Test cases for the `UserAgent` class
    """

    def test_create_useragent(self):
        """
        Test creating a user agent
        :return:
        :rtype:
        """

        obj = get_user_agent()

        self.assertEqual(first=obj.name, second=APP_NAME)
        self.assertEqual(first=obj.url, second=GITHUB_URL)
        self.assertEqual(first=obj.version, second=__version__)

    def test_useragent_str(self):
        """
        Test the string representation of the user agent
        :return:
        :rtype:
        """

        obj = get_user_agent()

        self.assertEqual(
            first=str(obj), second=f"{APP_NAME} ({GITHUB_URL}, {__version__})"
        )



"""
URL configuration for fleetpings app
"""

# Django
from django.urls import include, path

# AA Fleet Pings
from fleetpings import views
from fleetpings.constants import INTERNAL_URL_PREFIX

app_name: str = "fleetpings"

ajax_urls = path(
    route="ajax/",
    view=include(
        [
            path(
                route="create-fleet-ping/",
                view=views.ajax_create_fleet_ping,
                name="ajax_create_fleet_ping",
            ),
            path(
                route="get-ping-targets-for-user/",
                view=views.ajax_get_ping_targets,
                name="ajax_get_ping_targets",
            ),
            path(
                route="get-webhooks-for-user/",
                view=views.ajax_get_webhooks,
                name="ajax_get_webhooks",
            ),
            path(
                route="get-fleet-types-for-user/",
                view=views.ajax_get_fleet_types,
                name="ajax_get_fleet_types",
            ),
            path(
                route="get-formup-locations/",
                view=views.ajax_get_formup_locations,
                name="ajax_get_formup_locations",
            ),
            path(
                route="get-fleet-comms/",
                view=views.ajax_get_fleet_comms,
                name="ajax_get_fleet_comms",
            ),
            path(
                route="get-fleet-doctrines-for-user/",
                view=views.ajax_get_fleet_doctrines,
                name="ajax_get_fleet_doctrines",
            ),
        ]
    ),
)

urlpatterns = [
    path(route="", view=views.index, name="index"),
    # Ajax calls
    path(route=f"{INTERNAL_URL_PREFIX}/", view=include([ajax_urls])),
]



"""
Tests for model FormupLocation
"""

# Django
from django.test import TestCase

# AA Fleet Pings
from fleetpings.models import FormupLocation


class TestModelFormupLocation(TestCase):
    """
    Testing the FormupLocation model
    """

    def test_should_return_formuplocation_model_string_name(self):
        """
        Test should return FormupLocation model string name

        :return:
        :rtype:
        """

        test_object = FormupLocation(name="Alliance HQ")
        test_object.save()

        self.assertEqual(first=str(test_object), second="Alliance HQ")



"""
Tests for model DiscordPingTargets
"""

# Django
from django.contrib.auth.models import Group
from django.test import TestCase

# AA Fleet Pings
from fleetpings.models import DiscordPingTarget


class TestModelDiscordPingTarget(TestCase):
    """
    Testing the DiscordPingTarget model
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Setup

        :return:
        :rtype:
        """

        super().setUpClass()

        cls.group = Group.objects.create(name="Superhero")

    def test_should_return_pingtarget_model_string_name(self):
        """
        Test should return DiscordPingTarget model string name

        :return:
        :rtype:
        """

        test_object = DiscordPingTarget(name=self.group)

        self.assertEqual(first=str(test_object), second=self.group.name)



# Generated by Django 4.0.10 on 2023-09-22 19:00

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("fleetpings", "0014_update_models"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="discordpingtarget",
            options={
                "default_permissions": (),
                "verbose_name": "Discord ping target",
                "verbose_name_plural": "Discord ping targets",
            },
        ),
        migrations.AlterModelOptions(
            name="fleetcomm",
            options={
                "default_permissions": (),
                "verbose_name": "Fleet comm",
                "verbose_name_plural": "Fleet comms",
            },
        ),
        migrations.AlterModelOptions(
            name="fleetdoctrine",
            options={
                "default_permissions": (),
                "verbose_name": "Fleet doctrine",
                "verbose_name_plural": "Fleet doctrines",
            },
        ),
        migrations.AlterModelOptions(
            name="fleettype",
            options={
                "default_permissions": (),
                "verbose_name": "Fleet type",
                "verbose_name_plural": "Fleet types",
            },
        ),
        migrations.AlterModelOptions(
            name="formuplocation",
            options={
                "default_permissions": (),
                "verbose_name": "Formup location",
                "verbose_name_plural": "Formup locations",
            },
        ),
        migrations.AlterModelOptions(
            name="setting",
            options={
                "default_permissions": (),
                "verbose_name": "Setting",
                "verbose_name_plural": "Settings",
            },
        ),
        migrations.AlterField(
            model_name="fleettype",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict this fleet type to the following groups …",
                related_name="+",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.AlterField(
            model_name="setting",
            name="webhook_verification",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text="Whether to verify webhooks URLs or not. Note: When unchecked, webhook URLs will not be verified, so the app can be used with non-Discord webhooks as well. When disabling webhook verification and using non-Discord webhooks, it is up to you to make sure your webhook understands a payload that is formatted for Discord webhooks.",
                verbose_name="Verify webhooks",
            ),
        ),
    ]



"""
Handling Discord webhooks
"""

# Third Party
from dhooks_lite import Embed, Footer, UserAgent, Webhook

# Django
from django.contrib.auth.models import User
from django.utils import timezone

# AA Fleet Pings
from fleetpings import __version__
from fleetpings.constants import APP_NAME, GITHUB_URL
from fleetpings.helper.eve_images import get_character_portrait_from_evecharacter
from fleetpings.helper.ping_context import _get_webhook_ping_context


def get_user_agent() -> UserAgent:
    """
    Set the user agent

    :return: User agent
    :rtype: UserAgent
    """

    return UserAgent(APP_NAME, GITHUB_URL, __version__)


def ping_discord_webhook(ping_context: dict, user: User) -> None:
    """
    Sends a ping to a Discord webhook

    :param ping_context: The ping context
    :type ping_context: dict
    :param user: The user who sent the ping
    :type user: User
    :return:
    :rtype: None
    """

    webhook_ping_context = _get_webhook_ping_context(ping_context=ping_context)

    discord_webhook = Webhook(
        url=ping_context["ping_channel"]["webhook"], user_agent=get_user_agent()
    )
    message_to_send = webhook_ping_context["content"]
    embed_color = ping_context["ping_channel"]["embed_color"]
    author_eve_avatar = get_character_portrait_from_evecharacter(
        character=user.profile.main_character, size=256
    )
    author_eve_name = user.profile.main_character.character_name

    embed = Embed(
        description=message_to_send,
        title=".: Fleet Details :.",
        timestamp=timezone.now(),
        color=int(embed_color.lstrip("#"), 16),
        footer=Footer(
            text=f"Ping sent by: {author_eve_name}", icon_url=author_eve_avatar
        ),
    )

    discord_webhook.execute(
        content=webhook_ping_context["header"], embeds=[embed], wait_for_response=True
    )



"""
Test the settings
"""

# Django
from django.test import TestCase, modify_settings, override_settings

# AA Fleet Pings
from fleetpings.app_settings import (
    debug_enabled,
    discord_service_installed,
    fittings_installed,
    optimer_installed,
    srp_module_installed,
    srp_module_is,
    timezones_installed,
)


class TestSettings(TestCase):
    """
    Test the settings
    """

    @override_settings(DEBUG=True)
    def test_debug_enabled_with_debug_true(self) -> None:
        """
        Test debug_enabled with DEBUG = True

        :return:
        :rtype:
        """

        self.assertTrue(debug_enabled())

    @override_settings(DEBUG=False)
    def test_debug_enabled_with_debug_false(self) -> None:
        """
        Test debug_enabled with DEBUG = False

        :return:
        :rtype:
        """

        self.assertFalse(debug_enabled())

    @modify_settings(INSTALLED_APPS={"append": "timezones"})
    def test_timezones_installed(self) -> None:
        """
        Test timezones_installed with timezones installed

        :return:
        :rtype:
        """

        self.assertTrue(timezones_installed())

    @modify_settings(INSTALLED_APPS={"remove": "timezones"})
    def test_timezones_not_installed(self) -> None:
        """
        Test timezones_installed with timezones not installed

        :return:
        :rtype:
        """

        self.assertFalse(timezones_installed())

    @modify_settings(INSTALLED_APPS={"append": "allianceauth.optimer"})
    def test_optimer_installed(self) -> None:
        """
        Test optimer_installed with optimer installed

        :return:
        :rtype:
        """

        self.assertTrue(optimer_installed())

    @modify_settings(INSTALLED_APPS={"remove": "allianceauth.optimer"})
    def test_optimer_not_installed(self) -> None:
        """
        Test optimer_installed with optimer not installed

        :return:
        :rtype:
        """

        self.assertFalse(optimer_installed())

    @modify_settings(INSTALLED_APPS={"append": "fittings"})
    def test_fittings_installed(self) -> None:
        """
        Test fittings_installed with fittings installed

        :return:
        :rtype:
        """

        self.assertTrue(fittings_installed())

    @modify_settings(INSTALLED_APPS={"remove": "fittings"})
    def test_fittings_not_installed(self) -> None:
        """
        Test fittings_installed with fittings not installed

        :return:
        :rtype:
        """

        self.assertFalse(fittings_installed())

    @modify_settings(INSTALLED_APPS={"append": "allianceauth.srp"})
    @modify_settings(INSTALLED_APPS={"remove": "aasrp"})
    def test_srp_module_allianceauth_srp_installed(self) -> None:
        """
        Test srp_module_installed with allianceauth.srp installed

        :return:
        :rtype:
        """

        self.assertTrue(srp_module_installed())
        self.assertTrue(srp_module_is(module_name="allianceauth.srp"))
        self.assertFalse(srp_module_is(module_name="aasrp"))

    @modify_settings(INSTALLED_APPS={"remove": "allianceauth.srp"})
    @modify_settings(INSTALLED_APPS={"remove": "aasrp"})
    def test_srp_module_allianceauth_srp_not_installed(self) -> None:
        """
        Test srp_module_installed with allianceauth.srp not installed

        :return:
        :rtype:
        """

        self.assertFalse(srp_module_installed())

    @modify_settings(INSTALLED_APPS={"append": "aasrp"})
    @modify_settings(INSTALLED_APPS={"remove": "allianceauth.srp"})
    def test_srp_module_aasrp_installed(self) -> None:
        """
        Test srp_module_installed with aasrp installed

        :return:
        :rtype:
        """

        self.assertTrue(srp_module_installed())
        self.assertTrue(srp_module_is(module_name="aasrp"))
        self.assertFalse(srp_module_is(module_name="allianceauth.srp"))

    @modify_settings(INSTALLED_APPS={"remove": "aasrp"})
    @modify_settings(INSTALLED_APPS={"remove": "allianceauth.srp"})
    def test_srp_module_aasrp_not_installed(self) -> None:
        """
        Test srp_module_installed with aasrp not installed

        :return:
        :rtype:
        """

        self.assertFalse(srp_module_installed())

    @modify_settings(INSTALLED_APPS={"append": "allianceauth.services.modules.discord"})
    def test_discord_service_installed(self) -> None:
        """
        Test discord_service_installed with discord_service installed

        :return:
        :rtype:
        """

        self.assertTrue(discord_service_installed())

    @modify_settings(INSTALLED_APPS={"remove": "allianceauth.services.modules.discord"})
    def test_discord_service_not_installed(self) -> None:
        """
        Test discord_service_installed with discord_service not installed

        :return:
        :rtype:
        """

        self.assertFalse(discord_service_installed())



"""
Template tags
"""

# Django
from django.template.defaulttags import register

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag
from app_utils.urls import reverse_absolute

# AA Fleet Pings
from fleetpings import __title__

logger = LoggerAddTag(my_logger=get_extension_logger(__name__), prefix=__title__)


@register.simple_tag
def fleetpings_reverse_url(view: str, *args) -> str:
    """
    Reverse URL

    :param view:
    :type view:
    :param args:
    :type args:
    :return:
    :rtype:
    """

    return reverse_absolute(viewname=view, args=list(args))



"""
Tests for model Webhook
"""

# Standard Library
from unittest import mock

# Django
from django.core.exceptions import ValidationError
from django.test import TestCase

# AA Fleet Pings
from fleetpings.models import Webhook


class TestModelWebhook(TestCase):
    """
    Testing the Webhook model
    """

    def test_should_return_webhook_model_string_name(self):
        """
        Test should return Webhook model string name

        :return:
        :rtype:
        """

        test_object = Webhook(
            name="Test Webhook",
            url=(
                "https://discord.com/api/webhooks/754119343402302920F/x-BfFCdEG"
                "-qGg_39_mFUqRSLoz2dm6Oa8vxNdaAxZdgKOAyesVpy-Bzf8wDU_vHdFpm-"
            ),
        )

        test_object.save()

        self.assertEqual(first=str(test_object), second="Test Webhook")

    def test_discord_webhook_should_throw_exception(self):
        """
        Test if we get a ValidationError for an invalid Discord webhook

        :return:
        :rtype:
        """

        # given
        webhook = Webhook(
            url=(
                "https://discord.com/api/webhooks/754343402302911920F/x-BfFCdEG"
                "-qGg_39_mFwDU_vHdFUqRSLozaAxZdgKO2dm6Oa8vxNdAyesVpy-Bzf8pm-"
            ),
        )

        # when
        with self.assertRaises(expected_exception=ValidationError):
            webhook.clean()

        with self.assertRaisesMessage(
            expected_exception=ValidationError,
            expected_message=(
                "Invalid webhook URL. The webhook URL you entered does not match any "
                "known format for a Discord webhook. Please check the "
                "webhook URL."
            ),
        ):
            webhook.clean()

    @mock.patch(
        target="fleetpings.models.Setting.webhook_verification",
        new_callable=mock.PropertyMock,
    )
    def test_discord_webhook_should_not_throw_exception_with_verification_false(
        self, webhook_verification
    ):
        """
        Test we do not get a ValidationError for an invalid Discord webhook when
        settings.webhook_verification is set to False

        :param webhook_verification:
        :type webhook_verification:
        :return:
        :rtype:
        """

        # given
        webhook_verification.return_value = False

        # when
        webhook = Webhook(url="http://test/a/bad/webhook")

        # then
        self.assertIsNone(obj=webhook.clean())



# Generated by Django 4.0.10 on 2023-05-13 14:52

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        ("fleetpings", "0010_use_ellipsis_character_in_strings"),
    ]

    operations = [
        migrations.CreateModel(
            name="Setting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "use_default_fleet_types",
                    models.BooleanField(
                        db_index=True,
                        default=True,
                        help_text="Whether to use default fleet types. If checked, the default fleet types (Roaming, Home Defense, StratOP, and CTA) will be added to the Fleet Type dropdown.",
                        verbose_name="Use default fleet types",
                    ),
                ),
                (
                    "use_default_ping_targets",
                    models.BooleanField(
                        db_index=True,
                        default=True,
                        help_text="Whether to use default ping targets. If checked, the default ping targets (@everyone and @here) will be added to the Ping Target dropdown.",
                        verbose_name="Use default ping targets",
                    ),
                ),
                (
                    "use_doctrines_from_fittings_module",
                    models.BooleanField(
                        db_index=True,
                        default=False,
                        help_text="Whether to use the doctrines from the Fittings modules in the doctrine dropdown. Note: The fittings module needs to be installed for this.",
                        verbose_name="Use Doctrines from Fittings module",
                    ),
                ),
                (
                    "webhook_verification",
                    models.BooleanField(
                        db_index=True,
                        default=True,
                        help_text="Whether to verify Webhooks URLs or not. Note: When unchecked Webhook URLs will not be verified, so the app can be used with non-Discord Webhooks as well. When disabling webhook verification and using non-Discord webhooks, it is up to you to make sure your webhook understands a payload that is formatted for Discord webhooks.",
                        verbose_name="Verify Webhooks",
                    ),
                ),
                (
                    "default_embed_color",
                    models.CharField(
                        blank=True,
                        default="#FAA61A",
                        help_text="Default highlight color for the webhook embed.",
                        max_length=7,
                        verbose_name="Default embed color",
                    ),
                ),
            ],
            options={
                "verbose_name": "setting",
                "verbose_name_plural": "settings",
                "default_permissions": (),
            },
        ),
        migrations.AlterField(
            model_name="discordpingtargets",
            name="discord_id",
            field=models.CharField(
                blank=True,
                help_text="ID of the Discord role to ping",
                max_length=255,
                unique=True,
                verbose_name="Discord ID",
            ),
        ),
        migrations.AlterField(
            model_name="discordpingtargets",
            name="is_enabled",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text="Whether this formup location is enabled or not",
                verbose_name="Is enabled",
            ),
        ),
        migrations.AlterField(
            model_name="discordpingtargets",
            name="name",
            field=models.OneToOneField(
                help_text="Name of the Discord role to ping. (Note: This must be an Auth group that is synced to Discord.)",
                on_delete=django.db.models.deletion.CASCADE,
                to="auth.group",
                verbose_name="Group name",
            ),
        ),
        migrations.AlterField(
            model_name="discordpingtargets",
            name="notes",
            field=models.TextField(
                blank=True,
                help_text="You can add notes about this configuration here if you want",
                null=True,
                verbose_name="Notes",
            ),
        ),
        migrations.AlterField(
            model_name="discordpingtargets",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict ping rights to the following group(s) …",
                related_name="discord_role_require_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.AlterField(
            model_name="fleetcomm",
            name="is_enabled",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text="Whether this comms is enabled or not",
                verbose_name="Is enabled",
            ),
        ),
        migrations.AlterField(
            model_name="fleetcomm",
            name="name",
            field=models.CharField(
                help_text="Short name to identify this comms",
                max_length=255,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="fleetcomm",
            name="notes",
            field=models.TextField(
                blank=True,
                help_text="You can add notes about this configuration here if you want",
                null=True,
                verbose_name="Notes",
            ),
        ),
        migrations.AlterField(
            model_name="fleetdoctrine",
            name="is_enabled",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text="Whether this doctrine is enabled or not",
                verbose_name="Is enabled",
            ),
        ),
        migrations.AlterField(
            model_name="fleetdoctrine",
            name="link",
            field=models.CharField(
                blank=True,
                help_text="A link to a doctrine page for this doctrine if you have.",
                max_length=255,
                verbose_name="Doctrine link",
            ),
        ),
        migrations.AlterField(
            model_name="fleetdoctrine",
            name="name",
            field=models.CharField(
                help_text="Short name to identify this doctrine",
                max_length=255,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="fleetdoctrine",
            name="notes",
            field=models.TextField(
                blank=True,
                help_text="You can add notes about this configuration here if you want",
                null=True,
                verbose_name="Notes",
            ),
        ),
        migrations.AlterField(
            model_name="fleetdoctrine",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict this doctrine to the following group(s) …",
                related_name="fleetdoctrine_require_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.AlterField(
            model_name="fleettype",
            name="embed_color",
            field=models.CharField(
                blank=True,
                help_text="Highlight color for the embed",
                max_length=7,
                verbose_name="Embed color",
            ),
        ),
        migrations.AlterField(
            model_name="fleettype",
            name="is_enabled",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text="Whether this fleet type is enabled or not",
                verbose_name="Is enabled",
            ),
        ),
        migrations.AlterField(
            model_name="fleettype",
            name="name",
            field=models.CharField(
                help_text="Short name to identify this fleet type",
                max_length=255,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="fleettype",
            name="notes",
            field=models.TextField(
                blank=True,
                help_text="You can add notes about this configuration here if you want",
                null=True,
                verbose_name="Notes",
            ),
        ),
        migrations.AlterField(
            model_name="fleettype",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict this fleet type to the following group(s) …",
                related_name="fleettype_require_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.AlterField(
            model_name="formuplocation",
            name="is_enabled",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text="Whether this formup location is enabled or not",
                verbose_name="Is enabled",
            ),
        ),
        migrations.AlterField(
            model_name="formuplocation",
            name="name",
            field=models.CharField(
                help_text="Short name to identify this formup location",
                max_length=255,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="formuplocation",
            name="notes",
            field=models.TextField(
                blank=True,
                help_text="You can add notes about this configuration here if you want",
                null=True,
                verbose_name="Notes",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="is_enabled",
            field=models.BooleanField(
                db_index=True,
                default=True,
                help_text="Whether this webhook is active or not",
                verbose_name="Is enabled",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="name",
            field=models.CharField(
                help_text="Name of the channel this webhook posts to",
                max_length=255,
                unique=True,
                verbose_name="Discord channel",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="notes",
            field=models.TextField(
                blank=True,
                help_text="You can add notes about this webhook here if you want",
                null=True,
                verbose_name="Notes",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="restricted_to_group",
            field=models.ManyToManyField(
                blank=True,
                help_text="Restrict ping rights to the following group(s) …",
                related_name="webhook_require_groups",
                to="auth.group",
                verbose_name="Group restrictions",
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="url",
            field=models.CharField(
                help_text="URL of this webhook, e.g. https://discord.com/api/webhooks/123456/abcdef",
                max_length=255,
                unique=True,
                verbose_name="Webhook URL",
            ),
        ),
    ]



"""
Test ajax calls
"""

# Standard Library
from http import HTTPStatus

# Django
from django.contrib.auth.models import Group
from django.test import TestCase, modify_settings
from django.urls import reverse

# AA Fleet Pings
from fleetpings.tests.utils import create_fake_user


class TestAccess(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        """
        Setup

        :return:
        :rtype:
        """

        super().setUpClass()

        cls.group = Group.objects.create(name="Superhero")

        # User cannot access fleetpings
        cls.user_1001 = create_fake_user(
            character_id=1001, character_name="Peter Parker"
        )

        # User can access fleetpings
        cls.user_1002 = create_fake_user(
            character_id=1002,
            character_name="Bruce Wayne",
            permissions=["fleetpings.basic_access"],
        )

        # User can add srp (aasrp)
        cls.user_1003 = create_fake_user(
            character_id=1003,
            character_name="Clark Kent",
            permissions=[
                "fleetpings.basic_access",
                "aasrp.create_srp",
                "srp.add_srpfleetmain",
            ],
        )

    def test_ajax_get_ping_targets_no_access(self):
        """
        Test ajax call to get ping targets are not available for the current user
        without access to it

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1001)

        # when
        res = self.client.get(path=reverse(viewname="fleetpings:ajax_get_ping_targets"))

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)

    def test_ajax_get_ping_targets_general(self):
        """
        Test ajax call to get ping targets available for the current user

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1002)

        # when
        res = self.client.get(path=reverse(viewname="fleetpings:ajax_get_ping_targets"))

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.OK)

    def test_ajax_get_webhooks_no_access(self):
        """
        Test ajax call to get webhooks available for
        the current user without access to it

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1001)

        # when
        res = self.client.get(path=reverse(viewname="fleetpings:ajax_get_webhooks"))

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)

    def test_ajax_get_webhooks_general(self):
        """
        Test ajax call to get webhooks available for the current user

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1002)

        # when
        res = self.client.get(path=reverse(viewname="fleetpings:ajax_get_webhooks"))

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.OK)

    def test_ajax_get_fleet_types_no_access(self):
        """
        Test ajax call to get fleet types available for
        the current user without access to it

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1001)

        # when
        res = self.client.get(path=reverse(viewname="fleetpings:ajax_get_fleet_types"))

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)

    def test_ajax_get_fleet_types_general(self):
        """
        Test ajax call to get fleet types available for the current user

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1002)

        # when
        res = self.client.get(path=reverse(viewname="fleetpings:ajax_get_fleet_types"))

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.OK)

    def test_ajax_get_formup_locations_no_access(self):
        """
        Test ajax call to get formup locations available for
        a user without access to it

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1001)

        # when
        res = self.client.get(
            path=reverse(viewname="fleetpings:ajax_get_formup_locations")
        )

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)

    def test_ajax_get_formup_locations_general(self):
        """
        Test ajax call to get formup locations available for the current user

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1002)

        # when
        res = self.client.get(
            path=reverse(viewname="fleetpings:ajax_get_formup_locations")
        )

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.OK)

    def test_ajax_get_fleet_comms_no_access(self):
        """
        Test ajax call to get fleet comms available for
        a user without access to it

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1001)

        # when
        res = self.client.get(path=reverse(viewname="fleetpings:ajax_get_fleet_comms"))

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)

    def test_ajax_get_fleet_comms_general(self):
        """
        Test ajax call to get fleet comms available for the current user

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1002)

        # when
        res = self.client.get(path=reverse(viewname="fleetpings:ajax_get_fleet_comms"))

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.OK)

    def test_ajax_get_fleet_doctrines_no_access(self):
        """
        Test ajax call to get fleet doctrines available for
        a user without access to it

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1001)

        # when
        res = self.client.get(
            path=reverse(viewname="fleetpings:ajax_get_fleet_doctrines")
        )

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)

    def test_ajax_get_fleet_doctrines_general(self):
        """
        Test ajax call to get fleet doctrines available for the current user

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1002)

        # when
        res = self.client.get(
            path=reverse(viewname="fleetpings:ajax_get_fleet_doctrines")
        )

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.OK)

    def test_ajax_create_fleet_ping_no_access(self):
        """
        Test ajax call to create fleet pings available for
        a user without access to it

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1001)

        # when
        res = self.client.get(
            path=reverse(viewname="fleetpings:ajax_create_fleet_ping")
        )

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.FOUND)

    def test_ajax_create_fleet_ping_general(self):
        """
        Test ajax call to create fleet fleetpings for the current user

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1002)

        # when
        res = self.client.get(
            path=reverse(viewname="fleetpings:ajax_create_fleet_ping")
        )

        # then
        self.assertEqual(first=res.status_code, second=HTTPStatus.OK)

    def test_ajax_create_fleet_ping_general_with_form_data(self):
        """
        Test ajax call to create fleet pings for the current user with form data

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1002)
        form_data = {
            "ping_target": "@here",
            "pre_ping": 0,
            "ping_channel": "",
            "fleet_type": "CTA",
            "fleet_commander": "Jean Luc Picard",
            "fleet_name": "Starfleet",
            "formup_location": "Utopia Planitia",
            "formup_time": "",
            "formup_timestamp": "",
            "formup_now": 1,
            "fleet_comms": "Mumble",
            "fleet_doctrine": "Federation Ships",
            "fleet_doctrine_url": "",
            "webhook_embed_color": "",
            "srp": 1,
            "srp_link": 1,
            "additional_information": "Borg to slaughter!",
        }

        # when
        response = self.client.post(
            path=reverse(viewname="fleetpings:ajax_create_fleet_ping"), data=form_data
        )

        # then
        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)
        self.assertTemplateUsed(
            response=response,
            template_name="fleetpings/partials/ping/copy-paste-text.html",
        )
        self.assertContains(response=response, text="@here")
        self.assertContains(response=response, text="**FC:** Jean Luc Picard")
        self.assertContains(response=response, text="**Fleet Name:** Starfleet")
        self.assertContains(
            response=response, text="**Formup Location:** Utopia Planitia"
        )
        self.assertContains(response=response, text="**Comms:** Mumble")
        self.assertContains(
            response=response, text="**Ships / Doctrine:** Federation Ships"
        )
        self.assertContains(response=response, text="**SRP:** Yes")
        self.assertContains(response=response, text="Borg to slaughter!")

    @modify_settings(
        INSTALLED_APPS={
            "remove": ["allianceauth.srp", "aasrp"],  # Remove all SRP modules
            "append": "aasrp",  # Add allianceauth.srp
        }
    )
    def test_aasrp_link_creation(self):
        """
        Test if an SRP link is created when aasrp is installed

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1003)
        form_data = {
            "ping_target": "@here",
            "pre_ping": 0,
            "ping_channel": "",
            "fleet_type": "CTA",
            "fleet_commander": "Jean Luc Picard",
            "fleet_name": "Starfleet",
            "formup_location": "Utopia Planitia",
            "formup_time": "",
            "formup_timestamp": "",
            "formup_now": 1,
            "fleet_comms": "Mumble",
            "fleet_doctrine": "Federation Ships",
            "fleet_doctrine_url": "",
            "webhook_embed_color": "",
            "srp": 1,
            "srp_link": 1,
            "additional_information": "Borg to slaughter!",
        }

        # when
        response = self.client.post(
            path=reverse(viewname="fleetpings:ajax_create_fleet_ping"), data=form_data
        )

        # then
        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)
        self.assertContains(response=response, text="*SRP:** Yes")
        self.assertContains(response=response, text="SRP Code:")

    @modify_settings(
        INSTALLED_APPS={
            "remove": ["allianceauth.srp", "aasrp"],  # Remove all SRP modules
            "append": "allianceauth.srp",  # Add allianceauth.srp
        }
    )
    def test_allianceauth_srp_link_creation(self):
        """
        Test if an SRP link is created when allianceauth.srp is installed

        :return:
        :rtype:
        """

        # given
        self.client.force_login(user=self.user_1003)
        form_data = {
            "ping_target": "@here",
            "pre_ping": 0,
            "ping_channel": "",
            "fleet_type": "CTA",
            "fleet_commander": "Jean Luc Picard",
            "fleet_name": "Starfleet",
            "formup_location": "Utopia Planitia",
            "formup_time": "",
            "formup_timestamp": "",
            "formup_now": 1,
            "fleet_comms": "Mumble",
            "fleet_doctrine": "Federation Ships",
            "fleet_doctrine_url": "",
            "webhook_embed_color": "",
            "srp": 1,
            "srp_link": 1,
            "additional_information": "Borg to slaughter!",
        }

        # when
        response = self.client.post(
            path=reverse(viewname="fleetpings:ajax_create_fleet_ping"), data=form_data
        )

        # then
        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)
        self.assertContains(response=response, text="*SRP:** Yes")
        self.assertContains(response=response, text="SRP Code:")



"""
Form declarations
"""

# Django
from django import forms
from django.utils.text import format_lazy
from django.utils.translation import gettext_lazy as _

# AA Fleet Pings
from fleetpings.models import FleetType, Setting


def _get_discord_markdown_hint_text() -> str:
    """
    Get the Discord Markdown hint text

    :return:
    :rtype:
    """

    discord_helpdesk_url = (
        "https://support.discord.com/hc/en-us/articles/210298617"
        "-Markdown-Text-101-Chat-Formatting-Bold-Italic-Underline- "
    )

    discord_markdown_link_text = _("Discord Markdown")
    discord_markdown_link = (
        f'<a href="{discord_helpdesk_url}" target="_blank" rel="noopener noreferer">'
        f"{discord_markdown_link_text}</a>"
    )

    return format_lazy(
        _("Hint: You can use {discord_markdown_link} to format the text."),
        discord_markdown_link=discord_markdown_link,
    )


class FleetTypeAdminForm(forms.ModelForm):
    """
    Form definitions for the FleetType form in admin
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta
        """

        model = FleetType
        fields = "__all__"
        widgets = {"embed_color": forms.TextInput(attrs={"type": "color"})}


class SettingAdminForm(forms.ModelForm):
    """
    Form definitions for the Setting form in admin
    """

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta
        """

        model = Setting
        fields = "__all__"
        widgets = {"default_embed_color": forms.TextInput(attrs={"type": "color"})}


class FleetPingForm(forms.Form):
    """
    Form definitions for the FleetPing form
    """

    ping_target = forms.CharField(
        required=False,
        label=_("Ping target"),
        widget=forms.Select(choices={}),
        help_text=_("Who do you want to ping?"),
    )
    pre_ping = forms.BooleanField(
        initial=False,
        required=False,
        label=_("Pre-Ping"),
        help_text=_("Mark this checkbox if this should be a pre-ping."),
    )
    ping_channel = forms.CharField(
        required=False,
        label=_("Ping to"),
        widget=forms.Select(choices={}),
        help_text=_("Select a channel to ping automatically."),
    )
    fleet_type = forms.CharField(
        required=False, label=_("Fleet type"), widget=forms.Select(choices={})
    )
    fleet_commander = forms.CharField(
        required=False,
        label=_("FC name"),
        max_length=254,
        widget=forms.TextInput(attrs={"placeholder": _("Who is the FC?")}),
    )
    fleet_name = forms.CharField(
        required=False,
        label=_("Fleet name"),
        max_length=254,
        widget=forms.TextInput(
            attrs={
                "placeholder": _("What is the fleet name in the fleet finder in Eve?")
            }
        ),
    )
    formup_location = forms.CharField(
        required=False,
        label=_("Formup location"),
        widget=forms.TextInput(
            attrs={"data-datalist": "formup-location-list", "data-full-width": "true"}
        ),
    )
    formup_time = forms.CharField(
        required=False,
        label=_("Formup time"),
        max_length=254,
        widget=forms.TextInput(
            attrs={
                "disabled": "disabled",
                "autocomplete": "off",
                "placeholder": _("Formup time (EVE time)"),
            }
        ),
        help_text=_(
            "To enable this field, either make it a pre-ping (checkbox above) or "
            'uncheck "Formup NOW" (checkbox below).'
        ),
    )
    formup_timestamp = forms.CharField(
        required=False,
        label=_("Formup timestamp"),
        widget=forms.TextInput(attrs={"hidden": "hidden"}),
    )
    formup_now = forms.BooleanField(
        initial=True,
        required=False,
        label=_("Formup NOW"),
        help_text=_(
            'If this checkbox is active, formup time will be set to "NOW" '
            "and the time in the field above (if any is set) will be ignored."
        ),
    )
    fleet_comms = forms.CharField(
        required=False,
        label=_("Fleet comms"),
        widget=forms.TextInput(
            attrs={"data-datalist": "fleet-comms-list", "data-full-width": "true"}
        ),
    )
    fleet_doctrine = forms.CharField(
        required=False,
        label=_("Doctrine"),
        widget=forms.TextInput(
            attrs={"data-datalist": "fleet-doctrine-list", "data-full-width": "true"}
        ),
    )
    fleet_doctrine_url = forms.CharField(
        required=False,
        label=_("Doctrine link"),
        widget=forms.TextInput(attrs={"hidden": "hidden"}),
    )
    webhook_embed_color = forms.CharField(
        required=False,
        label=_("Webhook embed color"),
        widget=forms.TextInput(attrs={"hidden": "hidden"}),
    )
    srp = forms.BooleanField(
        initial=False,
        required=False,
        label=_("SRP"),
        help_text=_("Is this fleet covered by SRP?"),
    )
    srp_link = forms.BooleanField(
        initial=False,
        required=False,
        label=_("Create SRP link"),
        help_text=_(
            "If this checkbox is active, a SRP link specific for this fleet will be "
            "created.<br>Leave blank if unsure."
        ),
    )
    additional_information = forms.CharField(
        required=False,
        label=_("Additional information"),
        widget=forms.Textarea(
            attrs={
                "rows": 10,
                "cols": 20,
                "input_type": "textarea",
                "placeholder": _(
                    "Feel free to add some more information about the fleet …"
                ),
            }
        ),
        help_text=_get_discord_markdown_hint_text(),
    )
    optimer = forms.BooleanField(
        initial=False,
        required=False,
        label=_("Create Optimer"),
        help_text=_(
            "If this checkbox is active, a fleet operations timer for this pre-ping "
            "will be created."
        ),
    )
    fleet_duration = forms.CharField(
        required=False,
        label=_("Duration"),
        help_text=_("How long approximately will the fleet be?"),
    )
